use crate::{interpreter::Interpreter, value::Value};
use core::fmt::Debug;
use std::fmt::Display;
use thiserror::Error;

#[derive(Error, Debug)]
pub enum CallingError {
    #[error("Failed calling: {0}.")]
    CallFailed(String),
    #[error("Expected {0} arguments but got {1}.")]
    ArgumentMismatch(usize, usize),
    #[error("Failed calling: {0}.")]
    CallInterpretationFailed(String),
}

/// Interface for callable objects in Lox (functions, methods, classes).
pub trait Callable {
    fn name(&self) -> String;
    fn arity(&self) -> usize;
    fn call(&self, interpreter: &mut Interpreter, args: Vec<Value>) -> Result<Value, CallingError>;
}

impl Debug for dyn Callable {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self)
    }
}

impl Display for dyn Callable {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.name())
    }
}

impl PartialEq for dyn Callable {
    fn eq(&self, other: &Self) -> bool {
        self.name() == other.name() && self.arity() == other.arity()
    }
}
