//! Cli interface.

use std::path::PathBuf;

use clap::{Args, Parser, Subcommand};

/// Interpreter for the Lox language.
#[derive(Parser)]
#[command(version, about, long_about = None)]
#[command(propagate_version = true)]
pub struct Cli {
    #[command(subcommand)]
    pub command: Commands,
}

/// Configuration for running a Lox source file
#[derive(Args, Clone)]
pub struct RunConfig {
    /// Path to Lox source file.
    #[arg(short, long)]
    pub source: PathBuf,
}

/// Available commands for the Lox interpreter
#[derive(Subcommand)]
pub enum Commands {
    /// Run a Lox source file.
    Run(RunConfig),
    /// Run a Lox REPL.
    Repl,
}
