use crate::{
    keywords::KEYWORDS,
    token::{Token, TokenType},
};
use std::{iter::Peekable, str::CharIndices};

use super::interface::Tokenizer;

/// Consume an identifier which also might be a keyword.
///
/// An identifier starts with an alphabetic character or underscore and continues with
/// alphanumeric and underscore characters until the first different character.
pub struct Identifier;
impl Tokenizer for Identifier {
    fn run(
        &self,
        c: (usize, char),
        chars: &mut Peekable<CharIndices<'_>>,
        source: &str,
        line: usize,
    ) -> Option<(usize, Option<Token>)> {
        if c.1.is_alphabetic() || c.1 == '_' {
            let mut end_idx = c.0;
            while let Some((idx, _)) = chars.next_if(|(_, x)| x.is_alphanumeric() || c.1 == '_') {
                end_idx = idx;
            }

            let lexeme = source[c.0..=end_idx].to_string();
            let token_type = match KEYWORDS.get(&lexeme) {
                Some(token_type) => *token_type,
                None => TokenType::Identifier,
            };
            Some((
                0,
                Some(Token {
                    token_type,
                    lexeme,
                    literal: None,
                    line,
                }),
            ))
        } else {
            None
        }
    }
}
