use std::{iter::Peekable, str::CharIndices};

use crate::token::Token;

/// Interface to implement by a tokenizer. Each tokenizer handles a specific type of token
/// in the Lox language.
pub trait Tokenizer: Send + Sync {
    /// Take a tuple consisting of the index of a char and the char itself, the whole source code
    /// iterator, the source itself and the current line. Return None if you can not handle the
    /// current lexeme or an Option consisting of a tuple where the first element is how much the
    /// current line moved and the second element is an Option that can have the consumed token.
    fn run(
        &self,
        c: (usize, char),
        chars: &mut Peekable<CharIndices<'_>>,
        source: &str,
        line: usize,
    ) -> Option<(usize, Option<Token>)>;
}
