use super::interface::Tokenizer;
use crate::token::{
    Token,
    TokenType::{self, *},
};
use lazy_static::lazy_static;
use std::{collections::HashMap, iter::Peekable, str::CharIndices};

lazy_static! {
    /// Mapping of single characters to their respective TokenType.
    static ref SINGLE_CHAR_TOKENS: HashMap<char, TokenType> = {
        let mut m = HashMap::new();
        m.insert('(', LeftParen);
        m.insert(')', RightParen);
        m.insert('{', LeftBrace);
        m.insert('}', RightBrace);
        m.insert(',', Comma);
        m.insert('.', Dot);
        m.insert('-', Minus);
        m.insert('+', Plus);
        m.insert(';', Semicolon);
        m.insert('*', Star);
        m
    };
}

/// Consume a single character and produce its corresponding token.
pub struct SingleChar;
impl Tokenizer for SingleChar {
    fn run(
        &self,
        c: (usize, char),
        _chars: &mut Peekable<CharIndices<'_>>,
        source: &str,
        line: usize,
    ) -> Option<(usize, Option<Token>)> {
        let lexeme = source[c.0..=c.0].to_string();

        SINGLE_CHAR_TOKENS.get(&c.1).map(|token_type| {
            (
                0,
                Some(Token {
                    token_type: *token_type,
                    lexeme,
                    literal: None,
                    line,
                }),
            )
        })
    }
}
