use super::interface::Tokenizer;
use crate::token::{Literal, Token, TokenType};
use std::{iter::Peekable, str::CharIndices};
use tracing::error;

/// Consume a string literal from the source code.
///
/// A string literal consists of everything between two double quotes (")
/// and can stretch across multiple lines.
pub struct String;
impl Tokenizer for String {
    fn run(
        &self,
        c: (usize, char),
        chars: &mut Peekable<CharIndices<'_>>,
        source: &str,
        line: usize,
    ) -> Option<(usize, Option<Token>)> {
        match c.1 {
            '"' => {
                let mut lines = 0;
                while let Some((_, c)) = chars.next_if(|(_, peek)| *peek != '"') {
                    if c == '\n' {
                        lines += 1;
                    }
                }

                match chars.next_if(|(_, x)| *x == '"') {
                    Some((end_idx, _)) => {
                        let lexeme = source[c.0..=end_idx].to_string();
                        let literal = source[c.0 + 1..end_idx].to_string();

                        Some((
                            lines,
                            Some(Token {
                                token_type: TokenType::String,
                                lexeme,
                                literal: Some(Literal::String(literal)),
                                line,
                            }),
                        ))
                    }
                    None => {
                        error!("unterminated string");
                        None
                    }
                }
            }
            _ => None,
        }
    }
}
